==> gqmpeg_0.15.0.orig.tar.gz: gqmpeg-0.15.0/src/dndutil.c <==
/*
 * GQmpeg
 * (C) 2002 John Ellis
 *
 * Author: John Ellis
 *
 * This software is released under the GNU General Public License (GNU GPL).
 * Please read the included file COPYING for more information.
 * This software comes with no warranty of any kind, use at your own risk!
 */


#include "gqmpeg.h"
#include "dndutil.h"

#include "playlist.h"
#include "ui_fileops.h"


/*
 *-----------------------------------------------------------------------------
 * drag and drop uri utils
 *-----------------------------------------------------------------------------
 */

GList *uri_get_list(gchar *data)
{
	GList *list = NULL;
	gint b, e;

	b = e = 0;

	while (data[b] != '\0')
		{
		while (data[e] != '\r' && data[e] != '\n' && data[e] != '\0') e++;
		if (!strncmp(data + b, "file:", 5))
			{
			b += 5;
			while (data[b] == '/' && data[b+1] == '/') b++;
			list = g_list_append(list, g_strndup(data + b, e - b));
			}
		else if (!strncmp(data + b, "http:", 5))
			{
			list = g_list_append(list, g_strndup(data + b, e - b));
			}
		else if (!strncmp(data + b, "ftp:", 3))
			{
			list = g_list_append(list, g_strndup(data + b, e - b));
			}
		while (data[e] == '\r' || data[e] == '\n') e++;
		b = e;
		}

	return list;
}

gchar *uri_create_text(GList *list, gint *t, gint plain_text)
{
	GList *work;
	gchar *uri_text = NULL;
	gchar *leader = "file:";
	gchar *sep = "\r\n";
	gint total;
	gint leader_l;
	gint sep_l = strlen(sep);
	gchar *ptr;

	if (!list)
		{
		*t = 0;
		return NULL;
		}

	if (plain_text)
		{
		leader_l = 0;
		}
	else
		{
		leader_l = strlen(leader);
		}

	work = list;
	total = 0;

	/* compute length */

	while (work)
		{
		gchar *name = work->data;
		total += leader_l + strlen(name) + sep_l;
		work = work->next;
		}

	/* create list */
	uri_text = g_malloc(total + 1);
	ptr = uri_text;

	work = list;
	while (work)
		{
		gchar *name = work->data;
		if (leader_l > 0)
			{
			strcpy(ptr, leader);
			ptr += leader_l;
			}
		strcpy(ptr, name);
		ptr += strlen(name);
		strcpy(ptr, sep);
		ptr += sep_l;
		work = work->next;
		}
	ptr[0] = '\0';

	*t = total + 1;
	return uri_text;
}

void uri_parse_encoded_chars(GList *list)
{
	GList *work = list;

	while(work)
		{
		gchar *text = work->data;

		if (strchr(text, '%'))
			{
			gchar *w;
			gchar *r;

			w = r = text;

			while(*r != '\0')
				{
				if (*r == '%')
					{
					gchar t[3];

					r++;
					t[0] = *r;
					r++;
					t[1] = *r;
					t[2] = '\0';
					*w = (gchar)strtol(t, NULL, 16);
					}
				r++;
				w++;
				}
			if (*w != '\0') *w = '\0';
			}

		work = work->next;
		}
}

/*
 *-----------------------------------------------------------------------------
 * drag and drop for main window
 *-----------------------------------------------------------------------------
 */
enum {
	TARGET_URI_LIST,
	TARGET_X_URL,
	TARGET_NETSCAPE_URL
};

static GtkTargetEntry main_drop_types[] = {
	{ "text/uri-list", 0, TARGET_URI_LIST },
	{ "x-url/http",    0, TARGET_X_URL },
	{ "x-url/ftp",     0, TARGET_X_URL },
	{ "_NETSCAPE_URL", 0, TARGET_NETSCAPE_URL }
};
static gint main_drop_count = 4;

static void dnd_main_get_data(GtkWidget *widget,
			GdkDragContext *context, gint x, gint y,
			GtkSelectionData *selection_data, guint info,
			guint time, gpointer data)
{
	GList *list = NULL;

	switch (info)
		{
		case TARGET_URI_LIST:
		case TARGET_X_URL:
			list = uri_get_list(selection_data->data);
			break;
		case TARGET_NETSCAPE_URL:
			list = uri_get_list(selection_data->data);
			uri_parse_encoded_chars(list);
			break;
		default:
			list = NULL;
			break;
		}

	if (!list) return;

	if (!list->next)
		{
		gchar *name = list->data;
		if (isdir(name))
			{
			playlist_append_from_dir(name, FALSE);
			}
		else if (is_playlist(name))
			{
			playlist_load_from_file(name, FALSE, TRUE, TRUE);
			if (status == STATUS_STOP)
				{
				current_song_set_and_play(playlist_get_first(), NULL);
				}
			}
		else if (drop_single_to_playlist)
			{
			playlist_add(name, TRUE);
			if (status == STATUS_STOP)
				{
				current_song_set_and_play(playlist_get_count() - 1, NULL);
				}
			}
		else
			{
			current_song_set_and_play(-1, name);
			}
		}
	else
		{
		GList *work = list;
		while(work)
			{
			gchar *name = work->data;
			if (isdir(name))
				{
				playlist_append_from_dir(name, FALSE);
				}
			if (!is_playlist(name))
				{
				playlist_add(name, TRUE);
				}
			work = work->next;
			}
		}

	g_list_foreach(list, (GFunc)g_free, NULL);
	g_list_free(list);
}

void dnd_init(void)
{
	/* main window */
	gtk_drag_dest_set(main_window->window,
			GTK_DEST_DEFAULT_MOTION | GTK_DEST_DEFAULT_DROP,
			main_drop_types, main_drop_count, 
			GDK_ACTION_COPY);
	gtk_signal_connect(GTK_OBJECT(main_window->window), "drag_data_received",
			   GTK_SIGNAL_FUNC(dnd_main_get_data), NULL);
}

==> gqmpeg-0.16.0.tar.gz: gqmpeg-0.16.0/src/dndutil.c <==
/*
 * GQmpeg
 * (C) 2002 John Ellis
 *
 * Author: John Ellis
 *
 * This software is released under the GNU General Public License (GNU GPL).
 * Please read the included file COPYING for more information.
 * This software comes with no warranty of any kind, use at your own risk!
 */


#include "gqmpeg.h"
#include "dndutil.h"

#include "playlist.h"
#include "ui_fileops.h"


/*
 *-----------------------------------------------------------------------------
 * drag and drop uri utils
 *-----------------------------------------------------------------------------
 */

GList *uri_get_list(gchar *data)
{
	GList *list = NULL;
	gint b, e;

	b = e = 0;

	while (data[b] != '\0')
		{
		while (data[e] != '\r' && data[e] != '\n' && data[e] != '\0') e++;
		if (!strncmp(data + b, "file:", 5))
			{
			b += 5;
			while (data[b] == '/' && data[b+1] == '/') b++;
			list = g_list_append(list, g_strndup(data + b, e - b));
			}
		else if (!strncmp(data + b, "http:", 5))
			{
			list = g_list_append(list, g_strndup(data + b, e - b));
			}
		else if (!strncmp(data + b, "ftp:", 3))
			{
			list = g_list_append(list, g_strndup(data + b, e - b));
			}
		while (data[e] == '\r' || data[e] == '\n') e++;
		b = e;
		}

	return list;
}

gchar *uri_create_text(GList *list, gint *t, gint plain_text)
{
	GList *work;
	gchar *uri_text = NULL;
	gchar *leader = "file:";
	gchar *sep = "\r\n";
	gint total;
	gint leader_l;
	gint sep_l = strlen(sep);
	gchar *ptr;

	if (!list)
		{
		*t = 0;
		return NULL;
		}

	if (plain_text)
		{
		leader_l = 0;
		}
	else
		{
		leader_l = strlen(leader);
		}

	work = list;
	total = 0;

	/* compute length */

	while (work)
		{
		gchar *name = work->data;
		total += leader_l + strlen(name) + sep_l;
		work = work->next;
		}

	/* create list */
	uri_text = g_malloc(total + 1);
	ptr = uri_text;

	work = list;
	while (work)
		{
		gchar *name = work->data;
		if (leader_l > 0)
			{
			strcpy(ptr, leader);
			ptr += leader_l;
			}
		strcpy(ptr, name);
		ptr += strlen(name);
		strcpy(ptr, sep);
		ptr += sep_l;
		work = work->next;
		}
	ptr[0] = '\0';

	*t = total + 1;
	return uri_text;
}

void uri_parse_encoded_chars(GList *list)
{
	GList *work = list;

	while(work)
		{
		gchar *text = work->data;

		if (strchr(text, '%'))
			{
			gchar *w;
			gchar *r;

			w = r = text;

			while(*r != '\0')
				{
				if (*r == '%' && *(r + 1) != '\0' && *(r + 2) != '\0')
					{
					gchar t[3];
					gint n;

					r++;
					t[0] = *r;
					r++;
					t[1] = *r;
					t[2] = '\0';
					n = (gint)strtol(t, NULL, 16);
					if (n > 0 && n < 256)
						{
						*w = (gchar)n;
						}
					else
						{
						/* invalid number, rewind and ignore this escape */
						r -= 2;
						*w = *r;
						}
					}
				else if (w != r)
					{
					*w = *r;
					}
				r++;
				w++;
				}
			if (*w != '\0') *w = '\0';
			}

		work = work->next;
		}
}

/*
 *-----------------------------------------------------------------------------
 * drag and drop for main window
 *-----------------------------------------------------------------------------
 */
enum {
	TARGET_URI_LIST,
	TARGET_X_URL,
	TARGET_NETSCAPE_URL
};

static GtkTargetEntry main_drop_types[] = {
	{ "text/uri-list", 0, TARGET_URI_LIST },
	{ "x-url/http",    0, TARGET_X_URL },
	{ "x-url/ftp",     0, TARGET_X_URL },
	{ "_NETSCAPE_URL", 0, TARGET_NETSCAPE_URL }
};
static gint main_drop_count = 4;

static void dnd_main_get_data(GtkWidget *widget,
			GdkDragContext *context, gint x, gint y,
			GtkSelectionData *selection_data, guint info,
			guint time, gpointer data)
{
	GList *list = NULL;

	switch (info)
		{
		case TARGET_URI_LIST:
		case TARGET_X_URL:
			list = uri_get_list(selection_data->data);
			break;
		case TARGET_NETSCAPE_URL:
			list = uri_get_list(selection_data->data);
			uri_parse_encoded_chars(list);
			break;
		default:
			list = NULL;
			break;
		}

	if (!list) return;

	if (!list->next)
		{
		gchar *name = list->data;
		if (isdir(name))
			{
			playlist_append_from_dir(name, FALSE);
			}
		else if (is_playlist(name))
			{
			playlist_load_from_file(name, FALSE, TRUE, TRUE);
			if (status == STATUS_STOP)
				{
				current_song_set_and_play(playlist_get_first(), NULL);
				}
			}
		else if (drop_single_to_playlist)
			{
			playlist_add(name, TRUE);
			if (status == STATUS_STOP)
				{
				current_song_set_and_play(playlist_get_count() - 1, NULL);
				}
			}
		else
			{
			current_song_set_and_play(-1, name);
			}
		}
	else
		{
		GList *work = list;
		while(work)
			{
			gchar *name = work->data;
			if (isdir(name))
				{
				playlist_append_from_dir(name, FALSE);
				}
			if (!is_playlist(name))
				{
				playlist_add(name, TRUE);
				}
			work = work->next;
			}
		}

	g_list_foreach(list, (GFunc)g_free, NULL);
	g_list_free(list);
}

void dnd_init(void)
{
	/* main window */
	gtk_drag_dest_set(main_window->window,
			GTK_DEST_DEFAULT_MOTION | GTK_DEST_DEFAULT_DROP,
			main_drop_types, main_drop_count, 
			GDK_ACTION_COPY);
	gtk_signal_connect(GTK_OBJECT(main_window->window), "drag_data_received",
			   GTK_SIGNAL_FUNC(dnd_main_get_data), NULL);
}
